define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "epi/datetime",
    "epi/shell/TypeDescriptorManager",
    "epi/username",
    "epi-cms/ApplicationSettings",
    "./ActivityComment",
    "./viewmodels/ActivityViewModel",
    "./viewmodels/ActivityCommentViewModel",
    "./viewmodels/MessageActivityViewModel",
    // Parent class and mixins
    "dijit/layout/_LayoutWidget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "epi/shell/widget/_ModelBindingMixin",
    // Resources
    "dojo/text!./templates/Activity.html",
    "epi/i18n!epi/cms/nls/episerver.cms.activities.activity",
    //Widgets in the template
    "epi-cms/project/ActivityCommentForm"
], function(
    declare,
    lang,
    domClass,
    datetime,
    TypeDescriptorManager,
    username,
    ApplicationSettings,
    ActivityComment,
    ActivityViewModel,
    ActivityCommentViewModel,
    MessageActivityViewModel,
    // Parent class and mixins
    _LayoutWidget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _ModelBindingMixin,
    // Resources
    template,
    localizations
) {

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ModelBindingMixin], {
        // summary:
        //      A widget used to display an activity.
        // tags:
        //      internal

        // templateString: [protected] String
        //      A string that represents the widget template.
        templateString: template,

        // model: [protected] Object
        //      The view model which is a stateful object.
        model: null,

        // comments: [public] Array
        //      Array of comments on activity
        comments: null,

        // modelBindingMap: [protected] Object
        //      A map contain bindings from the model to properties on this object.
        modelBindingMap: {
            "comments": ["comments"]
        },

        buildRendering: function() {
            this.inherited(arguments);

            var activity = this.activity;

            var model =  new ActivityViewModel({
                id: activity.id,
                hasMessage: !!activity.message,
                comments: activity.comments ? activity.comments.slice() : []
            });

            this.commentForm.set("model", model);
            this.set("model", model);

            // Set the title to the name of the activity, e.g. project name or content name.
            var title = activity.name;

            // Append the language identifier to the title if the content language is different from
            // the editing language.
            var language = activity.language;
            if (language && language !== ApplicationSettings.currentContentLanguage) {
                title = lang.replace("{0} ({1})", [title, language]);
            }
            this.titleNode.textContent = title;

            // Set the icon class from the type identifier or default to the project icon.
            var iconClass = TypeDescriptorManager.getValue(activity.typeIdentifier, "iconClass");
            domClass.add(this.iconNode, iconClass || "epi-iconProject");

            // Set the color on the action node and the icon classes.
            var actionIconClass = lang.replace("epi-icon{0} epi-icon--{1}", [activity.actionIcon, activity.actionColor]);
            domClass.add(this.actionIconNode, actionIconClass);
            domClass.add(this.actionNode, "epi-activity--" + activity.actionColor);

            // Set the message based on the user and date updated.
            var template = activity.delayPublishUntil ? localizations.delaypublishmessagetemplate : localizations.messagetemplate;
            this.actionTextNode.textContent = lang.replace(template, {
                event: activity.actionText,
                delaypublish: datetime.toUserFriendlyString(activity.delayPublishUntil),
                user: username.toUserFriendlyString(activity.changedBy),
                date: datetime.toUserFriendlyString(activity.created)
            });

            // If the activity has a message property create an ActivityComment widget
            // and render the message
            if (activity.message) {
                domClass.add(this.domNode, "epi-activity--message");
                this.own(this._messageForm = new ActivityComment({ model: new MessageActivityViewModel(activity) }, this.messageNode));
            }

            // Create an ActivityComment widget for each comment
            if (activity.comments) {
                activity.comments.forEach(function(comment) {
                    this.addChild(new ActivityComment({ model: new ActivityCommentViewModel(comment) }));
                }, this);
            }
        },

        startup: function(){
            this.inherited(arguments);

            //Start the message comment form if it has been created
            this._messageForm && this._messageForm.startup();
        },

        _setCommentsAttr: function (comments) {
            // summary:
            //      Adds the new ActivityComment to the the Activity if a new comment has been added to the model
            // tags:
            //      private
            var hasActivityComments = (this.activity.comments ? this.activity.comments.length : 0);

            if(comments && comments.length != hasActivityComments) {
                var comment = comments[comments.length-1];
                this.addChild(new ActivityComment({ model: new ActivityCommentViewModel(comment) }));
            }
        }
    });
});
