define([], function(){

    return function(query, options){
        // summary:
        //      Determines if the given activity instance matches the query object
        // query: Object
        //
        // options: dojo/store/api/Store.QueryOptions?
        //  An object that contains optional information such as sort, start, and count.
        //
        // returns: Function
        //  A function that caches the passed query under the field "matches".  See any
        //  of the "query" methods on dojo.stores.
        //

        function matches(instance) {
            var projectId = query.projectId,
                references = query.contentReferences;

            if (projectId && references) {
                // The query matches the instance if they have the same project id or the instance
                // doesn't have a project id.
                var matchingProject = !instance.projectId || projectId === instance.projectId;
                return matchingProject && references.some(function (reference) {
                    return reference === instance.contentLink;
                });
            }
            return projectId === instance.projectId;
        }

        function execute(array) {

            // Clone the array so as not to modify the original object.
            var results = array.slice(),
                sortSet = options && options.sort;

            if(sortSet){
                results.sort(function(a, b){
                    for(var sort, i=0; sort = sortSet[i]; i++){
                        var aValue = a[sort.attribute],
                            bValue = b[sort.attribute];

                        // valueOf enables proper comparison of dates
                        aValue = aValue != null ? aValue.valueOf() : aValue;
                        bValue = bValue != null ? bValue.valueOf() : bValue;

                        if (aValue != bValue){
                            return sort.descending == (aValue == null || aValue > bValue) ? -1 : 1;
                        }
                    }
                    return 0;
                });
            }

            return results;
        }
        execute.matches = matches;

        return execute;
    };
});
